import pyperclip
from rich.console import Console

console = Console()

def copy_to_clipboard(text: str):
    """Copies the given text to the clipboard."""
    try:
        pyperclip.copy(text)
        console.print("[green]✔[/green] Enhanced prompt copied to clipboard.")
    except pyperclip.PyperclipException:
        console.print("[yellow]⚠[/yellow] Could not copy to clipboard. `xclip` or `xsel` may be required on Linux.")



import os
import yaml
from pathlib import Path
from typing import Dict, Any, Optional

DEFAULT_CONFIG = {
    "default_temperature": 0.7,
    "default_style": "detailed",
    "ollama_host": "http://localhost:11434",
    "timeout": 30,
    "max_tokens": 2000,
    "auto_copy": True,
    "display_colors": True,
    "auto_download_model": True,
    "enhancement_templates": {},
}

def get_config_path(config_path_str: Optional[str] = None) -> Path:
    if config_path_str:
        return Path(config_path_str)
    return Path.home() / ".enhance-this" / "config.yaml"

def load_config(config_path_str: Optional[str] = None) -> Dict[str, Any]:
    config_path = get_config_path(config_path_str)
    if not config_path.exists():
        return DEFAULT_CONFIG
    
    try:
        with open(config_path, 'r') as f:
            user_config = yaml.safe_load(f)
        
        config = DEFAULT_CONFIG.copy()
        if user_config:
            config.update(user_config)
        return config
    except (yaml.YAMLError, IOError):
        return DEFAULT_CONFIG

def ensure_config_dir_exists():
    config_path = get_config_path()
    config_path.parent.mkdir(parents=True, exist_ok=True)

def create_default_config_if_not_exists():
    ensure_config_dir_exists()
    config_path = get_config_path()
    if not config_path.exists():
        config_to_write = DEFAULT_CONFIG.copy()
        # Example for user
        config_to_write['enhancement_templates'] = {
            'my_style': '/path/to/your/custom_template.txt'
        }
        with open(config_path, 'w') as f:
            yaml.dump(config_to_write, f, default_flow_style=False, sort_keys=False)



import click
from rich.console import Console
from rich.markdown import Markdown
from rich.panel import Panel
from rich.live import Live
import sys
import difflib

from .config import load_config, create_default_config_if_not_exists
from .ollama_client import OllamaClient
from .enhancer import PromptEnhancer
from .clipboard import copy_to_clipboard

@click.command()
@click.argument('prompt', required=False)
@click.option('-m', '--model', 'model_name', help='Ollama model to use (auto-selects optimal if not specified)')
@click.option('-t', '--temperature', type=click.FloatRange(0.0, 2.0), help='Temperature for generation (0.0-2.0)')
@click.option('-l', '--length', 'max_tokens', type=int, help='Max tokens for enhancement')
@click.option('-c', '--config', 'config_path', type=click.Path(), help='Configuration file path')
@click.option('-v', '--verbose', is_flag=True, help='Enable verbose output')
@click.option('-n', '--no-copy', is_flag=True, help="Don't copy to clipboard")
@click.option('-o', '--output', 'output_file', type=click.File('w'), help='Save enhanced prompt to file')
@click.option('-s', '--style', type=click.Choice(['detailed', 'concise', 'creative', 'technical']), help='Enhancement style')
@click.option('--diff', is_flag=True, help='Show a diff between the original and enhanced prompt')
@click.option('--list-models', is_flag=True, help='List available Ollama models')
@click.option('--download-model', 'download_model_name', help='Download specific model from Ollama')
@click.option('--auto-setup', is_flag=True, help='Automatically setup Ollama with optimal model')
@click.option('--version', is_flag=True, help='Show version information')
@click.help_option('-h', '--help')
def main(prompt, model_name, temperature, max_tokens, config_path, verbose, no_copy, output_file, style, diff, list_models, download_model_name, auto_setup, version):
    """
    Enhances a simple prompt using Ollama AI models, displays the enhanced version,
    and automatically copies it to the clipboard.
    """
    if version:
        from importlib.metadata import version as get_version
        pkg_version = get_version("enhance-this")
        click.echo(f"enhance-this version {pkg_version}")
        return

    create_default_config_if_not_exists()
    config = load_config(config_path)
    
    console = Console()
    client = OllamaClient(host=config['ollama_host'], timeout=config['timeout'])

    if not client.is_running():
        console.print("[red]✖[/red] Ollama service is not running or is unreachable.")
        console.print("Please start Ollama and try again.")
        sys.exit(1)

    if list_models:
        models = client.list_models()
        if models:
            console.print("[bold green]Available Ollama models:[/bold green]")
            for model in models:
                console.print(f"- {model}")
        else:
            console.print("[yellow]No Ollama models found.[/yellow]")
        return

    if download_model_name:
        console.print(f"Starting download for '{download_model_name}'...")
        client.download_model(download_model_name)
        return
        
    available_models = client.list_models()

    if auto_setup or not available_models:
        if not available_models:
            console.print("[yellow]No models found. Starting auto-setup.[/yellow]")
        else:
            console.print("Starting auto-setup...")
        
        recommended_models = ["llama3.1:8b", "llama3", "mistral"]
        for model_to_try in recommended_models:
            if model_to_try not in available_models:
                console.print(f"Downloading recommended model: {model_to_try}")
                if client.download_model(model_to_try):
                    available_models.append(model_to_try)
                    break 
            else:
                console.print(f"Recommended model '{model_to_try}' is already available.")
                break
        else:
            console.print("[red]✖[/red] Auto-setup failed. Could not download a recommended model.")
            sys.exit(1)

        if auto_setup:
             return

    if not prompt:
        ctx = click.get_current_context()
        click.echo(ctx.get_help())
        ctx.exit()

    if model_name:
        if model_name not in available_models:
            console.print(f"[red]✖[/red] Model '{model_name}' not found. Available models:")
            for model in available_models:
                console.print(f"- {model}")
            sys.exit(1)
        final_model = model_name
    else:
        if not available_models:
            console.print("[red]✖[/red] No models available. Please download a model first, e.g.:")
            console.print("`enhance --download-model llama3.1:8b` or run `enhance --auto-setup`")
            sys.exit(1)
        final_model = available_models[0]
        if verbose:
            console.print(f"No model specified. Using first available model: [cyan]{final_model}[/cyan]")

    final_style = style or config.get('default_style', 'detailed')
    final_temperature = temperature if temperature is not None else config.get('default_temperature', 0.7)
    final_max_tokens = max_tokens or config.get('max_tokens', 2000)
    auto_copy_enabled = not no_copy and config.get('auto_copy', True)

    enhancer = PromptEnhancer(config.get('enhancement_templates'))
    system_prompt = enhancer.enhance(prompt, final_style)

    if verbose:
        console.print("\n[bold]System Prompt:[/bold]")
        console.print(Panel(system_prompt, title="System Prompt", border_style="dim"))

    enhanced_prompt = ""
    console.print("\n[bold magenta]✨ Enhanced Prompt ✨[/bold magenta]")

    status_obj = console.status("[bold green]Enhancing prompt...")
    status_obj.start()

    try:
        stream_generator = client.generate_stream(final_model, system_prompt, final_temperature, final_max_tokens)

        # Stop the status as soon as the generator is ready
        status_obj.stop()

        with Live(console=console, auto_refresh=False) as live:
            for chunk in stream_generator:
                enhanced_prompt += chunk
                live.update(Markdown(enhanced_prompt), refresh=True)
    finally:
        status_obj.stop()

    if enhanced_prompt:
        if diff:
            console.print("\n[bold yellow]↔️  Diff View ↔️[/bold yellow]")
            diff_result = difflib.unified_diff(
                prompt.splitlines(keepends=True),
                enhanced_prompt.splitlines(keepends=True),
                fromfile='Original',
                tofile='Enhanced',
            )
            for line in diff_result:
                if line.startswith('+'):
                    console.print(f"[green]{line}[/green]", end="")
                elif line.startswith('-'):
                    console.print(f"[red]{line}[/red]", end="")
                elif line.startswith('@'):
                    console.print(f"[dim]{line}[/dim]", end="")
                else:
                    console.print(line, end="")

        if output_file:
            output_file.write(enhanced_prompt)
            console.print(f"\n[green]✔[/green] Saved to {output_file.name}")

        if auto_copy_enabled:
            copy_to_clipboard(enhanced_prompt)
    else:
        console.print("\n[red]✖[/red] Failed to generate enhanced prompt.")
        sys.exit(1)

if __name__ == '__main__':
    main()



import pytest
from unittest.mock import patch
from enhance_this import clipboard
from rich.console import Console

@pytest.fixture
def mock_pyperclip_copy():
    with patch('pyperclip.copy') as mock_copy:
        yield mock_copy

@pytest.fixture
def mock_console_print():
    with patch.object(Console, 'print') as mock_print:
        yield mock_print

def test_copy_to_clipboard_success(mock_pyperclip_copy, mock_console_print):
    test_text = "This is a test string"
    clipboard.copy_to_clipboard(test_text)
    mock_pyperclip_copy.assert_called_once_with(test_text)
    mock_console_print.assert_called_once_with("[green]✔[/green] Enhanced prompt copied to clipboard.")

def test_copy_to_clipboard_failure(mock_pyperclip_copy, mock_console_print):
    mock_pyperclip_copy.side_effect = clipboard.pyperclip.PyperclipException("Copy failed")
    test_text = "This is a test string"
    clipboard.copy_to_clipboard(test_text)
    mock_pyperclip_copy.assert_called_once_with(test_text)
    mock_console_print.assert_called_once_with("[yellow]⚠[/yellow] Could not copy to clipboard. `xclip` or `xsel` may be required on Linux.")



import requests
import json
from rich.console import Console
from rich.progress import Progress, SpinnerColumn, BarColumn, TextColumn
from typing import List, Dict, Any, Iterator
from requests.adapters import HTTPAdapter, Retry

console = Console()

class OllamaClient:
    def __init__(self, host: str, timeout: int):
        self.host = host
        self.timeout = timeout
        self.session = requests.Session()
        retries = Retry(total=3, backoff_factor=1, status_forcelist=[500, 502, 503, 504])
        self.session.mount('http://', HTTPAdapter(max_retries=retries))
        self.session.mount('https://', HTTPAdapter(max_retries=retries))

    def is_running(self) -> bool:
        try:
            response = self.session.get(self.host, timeout=self.timeout)
            return response.status_code == 200
        except requests.RequestException:
            return False

    def list_models(self) -> List[str]:
        try:
            response = self.session.get(f"{self.host}/api/tags", timeout=self.timeout)
            response.raise_for_status()
            models = response.json().get("models", [])
            return [model["name"] for model in models]
        except requests.RequestException:
            return []

    def download_model(self, model_name: str) -> bool:
        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            BarColumn(),
            TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
            transient=True,
        ) as progress:
            task = progress.add_task(f"[cyan]Downloading {model_name}", total=None)
            try:
                response = self.session.post(
                    f"{self.host}/api/pull",
                    json={"name": model_name, "stream": True},
                    stream=True,
                    timeout=None  # No timeout for download
                )
                response.raise_for_status()
                
                total = 0
                for line in response.iter_lines():
                    if line:
                        data = json.loads(line)
                        if "total" in data and "completed" in data:
                            if not progress.tasks[task].total:
                                progress.update(task, total=data["total"])
                            progress.update(task, completed=data["completed"])
                        if data.get("status") == "success":
                            if progress.tasks[task].total:
                                progress.update(task, completed=progress.tasks[task].total)
                            break
                console.print(f"[green]✔[/green] Model '{model_name}' downloaded successfully.")
                return True
            except requests.RequestException as e:
                console.print(f"[red]✖[/red] Failed to download model '{model_name}': {e}")
                return False
            except json.JSONDecodeError:
                console.print(f"[red]✖[/red] Failed to parse response from Ollama while downloading.")
                return False

    def generate_stream(self, model: str, prompt: str, temperature: float, max_tokens: int) -> Iterator[str]:
        try:
                response = self.session.post(
                    f"{self.host}/api/generate",
                    json={
                        "model": model,
                        "prompt": prompt,
                        "stream": True,
                        "options": {
                            "temperature": temperature,
                            "num_predict": max_tokens,
                        }
                    },
                    stream=True,
                    timeout=self.timeout,
                )
                response.raise_for_status()
                for line in response.iter_lines():
                    if line:
                        data = json.loads(line)
                        yield data.get("response", "")
                        if data.get("done"):
                            break
        except requests.exceptions.Timeout:
            console.print(f"[red]✖[/red] Ollama request timed out after {self.timeout} seconds.")
        except requests.RequestException as e:
            console.print(f"[red]✖[/red] Error communicating with Ollama: {e}")



import pytest
import os
import yaml
from pathlib import Path
from enhance_this import config

@pytest.fixture
def mock_config_path(tmp_path):
    # Create a temporary config directory and file for testing
    test_config_dir = tmp_path / ".enhance-this"
    test_config_dir.mkdir()
    test_config_file = test_config_dir / "config.yaml"
    
    # Patch get_config_path to return our temporary path
    original_get_config_path = config.get_config_path
    config.get_config_path = lambda *args, **kwargs: test_config_file
    yield test_config_file
    config.get_config_path = original_get_config_path # Restore original

def test_create_default_config_if_not_exists(mock_config_path):
    # Ensure config file does not exist initially
    assert not mock_config_path.exists()

    config.create_default_config_if_not_exists()
    assert mock_config_path.exists()

    loaded_config = yaml.safe_load(mock_config_path.read_text())
    assert loaded_config["default_temperature"] == 0.7
    assert "my_style" in loaded_config["enhancement_templates"]

def test_load_config_default(mock_config_path):
    # No config file exists, should load default config
    loaded_config = config.load_config()
    assert loaded_config["default_temperature"] == 0.7
    assert loaded_config["ollama_host"] == "http://localhost:11434"
    assert loaded_config["enhancement_templates"] == {}

def test_load_config_with_existing_file(mock_config_path):
    # Create a custom config file
    custom_settings = {
        "default_temperature": 0.9,
        "ollama_host": "http://my-ollama:11434",
        "new_setting": "value",
        "enhancement_templates": {"custom": "/tmp/custom.txt"}
    }
    mock_config_path.write_text(yaml.dump(custom_settings))

    loaded_config = config.load_config()
    assert loaded_config["default_temperature"] == 0.9
    assert loaded_config["ollama_host"] == "http://my-ollama:11434"
    assert loaded_config["new_setting"] == "value"
    assert loaded_config["enhancement_templates"] == {"custom": "/tmp/custom.txt"}

def test_load_config_with_invalid_file(mock_config_path):
    # Create an invalid config file
    mock_config_path.write_text("invalid yaml: - ")

    loaded_config = config.load_config()
    # Should fall back to default config
    assert loaded_config["default_temperature"] == 0.7
    assert loaded_config["ollama_host"] == "http://localhost:11434"



import pytest
from unittest.mock import patch, mock_open
from enhance_this.enhancer import PromptEnhancer, load_templates
from pathlib import Path

# Mock importlib.resources.files for Python 3.9+
@pytest.fixture
def mock_importlib_resources_files():
    with patch('importlib.resources.files') as mock_files:
        # Create a mock directory structure
        mock_template_dir = {
            'detailed.txt': 'Detailed template: {user_prompt}',
            'concise.txt': 'Concise template: {user_prompt}',
            'creative.txt': 'Creative template: {user_prompt}',
            'technical.txt': 'Technical template: {user_prompt}',
        }

        def mock_path_object(name):
            mock_path = Path(name) # Use a real Path object for some methods
            mock_path.is_file = lambda: name in mock_template_dir
            mock_path.read_text = lambda encoding='utf-8': mock_template_dir[name]
            return mock_path

        mock_files.return_value = type('MockPath', (object,), {
            '__truediv__': lambda self, name: mock_path_object(name),
            'is_file': lambda: True, # For the directory itself
        })()
        yield

# Mock importlib.resources.open_text for Python 3.8
@pytest.fixture
def mock_importlib_resources_open_text():
    with patch('importlib.resources.open_text') as mock_open_text:
        def _mock_open_text(package, filename):
            content = {
                'detailed.txt': 'Detailed template: {user_prompt}',
                'concise.txt': 'Concise template: {user_prompt}',
                'creative.txt': 'Creative template: {user_prompt}',
                'technical.txt': 'Technical template: {user_prompt}',
            }.get(filename)
            if content is None:
                raise FileNotFoundError
            mock_file = mock_open(read_data=content)()
            return mock_file
        mock_open_text.side_effect = _mock_open_text
        yield

@pytest.fixture(autouse=True)
def mock_template_loading(mock_importlib_resources_files, mock_importlib_resources_open_text):
    # This fixture ensures that template loading is mocked for all tests in this module
    pass

def test_load_templates_builtin():
    templates = load_templates()
    assert "detailed" in templates
    assert "concise" in templates
    assert "creative" in templates
    assert "technical" in templates
    assert templates["detailed"] == "Detailed template: {user_prompt}"

def test_load_templates_with_custom_templates(tmp_path):
    custom_template_file = tmp_path / "my_custom_template.txt"
    custom_template_file.write_text("My custom template: {user_prompt}")

    custom_template_paths = {
        "custom_style": str(custom_template_file),
        "detailed": str(custom_template_file), # Override built-in
        "non_existent": str(tmp_path / "non_existent.txt") # Non-existent path
    }

    templates = load_templates(custom_template_paths)
    assert "custom_style" in templates
    assert templates["custom_style"] == "My custom template: {user_prompt}"
    assert templates["detailed"] == "My custom template: {user_prompt}" # Check override
    assert "non_existent" not in templates # Non-existent should not be loaded

def test_prompt_enhancer_enhance():
    enhancer = PromptEnhancer()
    enhanced_prompt = enhancer.enhance("my simple prompt", "detailed")
    assert enhanced_prompt == "Detailed template: my simple prompt"

def test_prompt_enhancer_unknown_style():
    enhancer = PromptEnhancer()
    with pytest.raises(ValueError, match="Unknown style: unknown_style"):
        enhancer.enhance("my simple prompt", "unknown_style")

def test_load_templates_with_invalid_custom_path_type():
    custom_template_paths = {
        "invalid_style": 123 # Not a string
    }
    templates = load_templates(custom_template_paths)
    assert "invalid_style" not in templates

def test_load_templates_with_empty_custom_path_string():
    custom_template_paths = {
        "empty_path_style": "" # Empty string
    }
    templates = load_templates(custom_template_paths)
    assert "empty_path_style" not in templates



import importlib.resources
from typing import Dict, Optional
from pathlib import Path
from rich.console import Console

console = Console()

def load_templates(custom_template_paths: Optional[Dict[str, str]] = None) -> Dict[str, str]:
    templates = {}
    package = 'enhance_this.templates'
    
    # Load built-in templates
    try:
        from importlib.resources import files
    except ImportError:
        from importlib_resources import files

    try:
        template_dir = files(package)
        for style in ["detailed", "concise", "creative", "technical"]:
            template_file = template_dir / f"{style}.txt"
            if template_file.is_file():
                 templates[style] = template_file.read_text(encoding='utf-8')
    except Exception:
        for style in ["detailed", "concise", "creative", "technical"]:
            try:
                with importlib.resources.open_text(package, f"{style}.txt") as f:
                    templates[style] = f.read()
            except FileNotFoundError:
                pass

    # Load custom templates from config
    if custom_template_paths:
        for style, path_str in custom_template_paths.items():
            if not isinstance(path_str, str) or not path_str.strip():
                continue # Skip if path is not a valid string
            try:
                path = Path(path_str).expanduser()
                if path.is_file():
                    templates[style] = path.read_text(encoding='utf-8')
                else:
                    console.print(f"[yellow]⚠[/yellow] Custom template for style '{style}' not found at: {path_str}")
            except Exception as e:
                console.print(f"[red]✖[/red] Error loading custom template for style '{style}': {e}")

    return templates

class PromptEnhancer:
    def __init__(self, custom_template_paths: Optional[Dict[str, str]] = None):
        self.templates = load_templates(custom_template_paths)

    def enhance(self, user_prompt: str, style: str) -> str:
        if style not in self.templates:
            available_styles = list(self.templates.keys())
            raise ValueError(f"Unknown style: '{style}'. Available styles: {available_styles}")
        
        template = self.templates[style]
        return template.format(user_prompt=user_prompt)



import pytest
from click.testing import CliRunner
from unittest.mock import patch, MagicMock
from enhance_this.cli import main
from enhance_this import config

@pytest.fixture
def runner():
    return CliRunner()

@pytest.fixture(autouse=True)
def mock_ollama_client():
    with patch('enhance_this.ollama_client.OllamaClient') as MockOllamaClient:
        instance = MockOllamaClient.return_value
        instance.is_running.return_value = True
        instance.list_models.return_value = ["llama2"]
        instance.download_model.return_value = True
        instance.generate_stream.return_value = iter(["Enhanced ", "Prompt"])
        yield instance

@pytest.fixture(autouse=True)
def mock_clipboard():
    with patch('enhance_this.clipboard.copy_to_clipboard') as mock_copy:
        yield mock_copy

@pytest.fixture(autouse=True)
def mock_config_file(tmp_path):
    # Create a temporary config directory and file for testing
    test_config_dir = tmp_path / ".enhance-this"
    test_config_dir.mkdir()
    test_config_file = test_config_dir / "config.yaml"
    
    # Patch get_config_path to return our temporary path
    original_get_config_path = config.get_config_path
    config.get_config_path = lambda *args, **kwargs: test_config_file
    yield test_config_file
    config.get_config_path = original_get_config_path # Restore original

    # Ensure a default config is created for tests that don't explicitly create one
    config.create_default_config_if_not_exists()

def test_cli_version(runner):
    result = runner.invoke(main, ["--version"])
    assert result.exit_code == 0
    assert "enhance-this version" in result.output

def test_cli_help(runner):
    result = runner.invoke(main, ["--help"])
    assert result.exit_code == 0
    assert "Usage: enhance [OPTIONS] [PROMPT]" in result.output

def test_cli_no_prompt(runner):
    result = runner.invoke(main, [])
    assert result.exit_code == 0 # Help message is shown, not an error
    assert "Usage: enhance [OPTIONS] [PROMPT]" in result.output

def test_cli_ollama_not_running(runner, mock_ollama_client):
    mock_ollama_client.is_running.return_value = False
    result = runner.invoke(main, ["test prompt"])
    assert result.exit_code == 1
    assert "Ollama service is not running or is unreachable." in result.output

def test_cli_list_models(runner, mock_ollama_client):
    mock_ollama_client.list_models.return_value = ["model1", "model2"]
    result = runner.invoke(main, ["--list-models"])
    assert result.exit_code == 0
    assert "Available Ollama models:" in result.output
    assert "- model1" in result.output
    assert "- model2" in result.output

def test_cli_download_model(runner, mock_ollama_client):
    result = runner.invoke(main, ["--download-model", "new_model"])
    assert result.exit_code == 0
    mock_ollama_client.download_model.assert_called_once_with("new_model")
    assert "Starting download for 'new_model'..." in result.output

def test_cli_auto_setup_no_models(runner, mock_ollama_client):
    mock_ollama_client.list_models.return_value = []
    result = runner.invoke(main, ["--auto-setup"])
    assert result.exit_code == 0
    mock_ollama_client.download_model.assert_called_once_with("llama3.1:8b")
    assert "No models found. Starting auto-setup." in result.output

def test_cli_auto_setup_model_exists(runner, mock_ollama_client):
    mock_ollama_client.list_models.return_value = ["llama3.1:8b"]
    result = runner.invoke(main, ["--auto-setup"])
    assert result.exit_code == 0
    mock_ollama_client.download_model.assert_not_called()
    assert "Recommended model 'llama3.1:8b' is already available." in result.output

def test_cli_enhance_success(runner, mock_ollama_client, mock_clipboard):
    result = runner.invoke(main, ["my test prompt"])
    assert result.exit_code == 0
    assert "✨ Enhanced Prompt ✨" in result.output
    assert "Enhanced Prompt" in result.output # From the streamed output
    mock_ollama_client.generate_stream.assert_called_once()
    mock_clipboard.assert_called_once_with("Enhanced Prompt")

def test_cli_enhance_no_copy(runner, mock_ollama_client, mock_clipboard):
    result = runner.invoke(main, ["my test prompt", "-n"])
    assert result.exit_code == 0
    mock_clipboard.assert_not_called()

def test_cli_enhance_output_file(runner, mock_ollama_client, tmp_path):
    output_file = tmp_path / "output.txt"
    result = runner.invoke(main, ["my test prompt", "-o", str(output_file)])
    assert result.exit_code == 0
    assert f"Saved to {output_file.name}" in result.output
    assert output_file.read_text() == "Enhanced Prompt"

def test_cli_enhance_diff_view(runner, mock_ollama_client):
    mock_ollama_client.generate_stream.return_value = iter(["This is the ", "enhanced output."])
    result = runner.invoke(main, ["This is the original input.", "--diff"])
    assert result.exit_code == 0
    assert "↔️  Diff View ↔️" in result.output
    assert "- This is the original input." in result.output
    assert "+ This is the enhanced output." in result.output

def test_cli_enhance_verbose(runner, mock_ollama_client):
    result = runner.invoke(main, ["my test prompt", "-v"])
    assert result.exit_code == 0
    assert "System Prompt:" in result.output
    mock_ollama_client.generate_stream.assert_called_once()

def test_cli_enhance_specific_model_not_found(runner, mock_ollama_client):
    mock_ollama_client.list_models.return_value = ["model_a"]
    result = runner.invoke(main, ["prompt", "-m", "non_existent_model"])
    assert result.exit_code == 1
    assert "Model 'non_existent_model' not found." in result.output

def test_cli_enhance_no_models_available(runner, mock_ollama_client):
    mock_ollama_client.list_models.return_value = []
    mock_ollama_client.download_model.return_value = False # Simulate download failure
    result = runner.invoke(main, ["prompt"])
    assert result.exit_code == 1
    assert "No models available. Please download a model first" in result.output

def test_cli_enhance_custom_style(runner, mock_ollama_client, mock_config_file):
    # Create a custom template file
    custom_template_path = mock_config_file.parent / "custom_template.txt"
    custom_template_path.write_text("Custom template for: {user_prompt}")

    # Update the config file to include the custom template
    with open(mock_config_file, 'a') as f:
        f.write("\nenhancement_templates:\n  my_custom_style: " + str(custom_template_path) + "\n")

    result = runner.invoke(main, ["my prompt", "-s", "my_custom_style"])
    assert result.exit_code == 0
    # The enhancer will use the custom template, so the generated prompt will reflect that
    mock_ollama_client.generate_stream.assert_called_once()
    # We need to mock the enhancer's behavior to reflect the custom template
    # For now, we'll just check that the command ran successfully.
    assert "✨ Enhanced Prompt ✨" in result.output




import pytest
from unittest.mock import patch, MagicMock
from enhance_this.ollama_client import OllamaClient
import requests
import json

@pytest.fixture
def mock_requests_session():
    with patch('requests.Session') as mock_session_class:
        mock_session_instance = MagicMock()
        mock_session_class.return_value = mock_session_instance
        yield mock_session_instance

@pytest.fixture
def ollama_client(mock_requests_session):
    return OllamaClient(host="http://localhost:11434", timeout=5)

def test_is_running_success(ollama_client, mock_requests_session):
    mock_requests_session.get.return_value.status_code = 200
    assert ollama_client.is_running()

def test_is_running_failure(ollama_client, mock_requests_session):
    mock_requests_session.get.side_effect = requests.RequestException
    assert not ollama_client.is_running()

def test_list_models_success(ollama_client, mock_requests_session):
    mock_response = MagicMock()
    mock_response.json.return_value = {"models": [{"name": "llama2"}, {"name": "mistral"}]}
    mock_requests_session.get.return_value = mock_response
    models = ollama_client.list_models()
    assert models == ["llama2", "mistral"]

def test_list_models_failure(ollama_client, mock_requests_session):
    mock_requests_session.get.side_effect = requests.RequestException
    models = ollama_client.list_models()
    assert models == []

def test_download_model_success(ollama_client, mock_requests_session):
    mock_response = MagicMock()
    mock_response.iter_lines.return_value = [
        json.dumps({"status": "downloading", "total": 100, "completed": 50}).encode(),
        json.dumps({"status": "success", "total": 100, "completed": 100}).encode(),
    ]
    mock_requests_session.post.return_value = mock_response
    assert ollama_client.download_model("llama2")

def test_download_model_failure(ollama_client, mock_requests_session):
    mock_requests_session.post.side_effect = requests.RequestException
    assert not ollama_client.download_model("llama2")

def test_generate_stream_success(ollama_client, mock_requests_session):
    mock_response = MagicMock()
    mock_response.iter_lines.return_value = [
        json.dumps({"response": "Hello "}).encode(),
        json.dumps({"response": "World!", "done": True}).encode(),
    ]
    mock_requests_session.post.return_value = mock_response
    
    chunks = list(ollama_client.generate_stream("llama2", "prompt", 0.7, 200))
    assert chunks == ["Hello ", "World!"]

def test_generate_stream_timeout(ollama_client, mock_requests_session):
    mock_requests_session.post.side_effect = requests.exceptions.Timeout
    chunks = list(ollama_client.generate_stream("llama2", "prompt", 0.7, 200))
    assert chunks == []

def test_generate_stream_request_exception(ollama_client, mock_requests_session):
    mock_requests_session.post.side_effect = requests.RequestException
    chunks = list(ollama_client.generate_stream("llama2", "prompt", 0.7, 200))
    assert chunks == []

# Integration Test (requires Ollama running with llama2 model)
@pytest.mark.skip(reason="Integration test: requires Ollama running with llama2 model")
def test_integration_generate_stream_live():
    client = OllamaClient(host="http://localhost:11434", timeout=60) # Increased timeout for live test
    if not client.is_running():
        pytest.skip("Ollama is not running.")
    
    # Ensure llama2 is available, or download it
    if "llama2" not in client.list_models():
        print("\nllama2 model not found, attempting to download...")
        if not client.download_model("llama2"):
            pytest.fail("Failed to download llama2 model for integration test.")

    prompt = "Say hello world"
    full_response = "".join(list(client.generate_stream("llama2", prompt, 0.7, 50)))
    assert "hello world" in full_response.lower()
